﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace JsDraw
{
    //=========================================================================
    /// <summary>
    /// ログ書き込みクラス
    /// </summary>
    public class LogWriter
    {
        #region ＡＰＩ定義
        #endregion

        #region 定数、列挙体定義

        /// <summary>最大ログファイルサイズ</summary>
        private const int MAX_LOG_FILE_SIZE = 1024000;

        #endregion

        #region イベント定義
        #endregion

        #region メンバ変数

        /// <summary>初期化フラグ</summary>
        static protected bool _initFlag;

        /// <summary>ログファイル名</summary>
        static protected string _logFileName;

        #endregion

        #region コンストラクタ、デストラクタ

        //=====================================================================
        /// <summary>
        /// コンストラクタ
        /// </summary>
        static LogWriter()
        {
            // メンバ変数初期化
            _initFlag = false;
            _logFileName = string.Empty;
        }

        #endregion

        #region パブリックプロパティ

        //=====================================================================
        /// <summary>
        /// 初期化処理
        /// <param name="logFileName">ログファイル名</param>
        /// </summary>
        public static void Initialize(string logFileName)
        {
            // 初期化フラグ設定
            _initFlag = true;

            // ログファイル名設定
            _logFileName = logFileName;

        }
        
        #endregion

        #region パブリックメソッド
        //=====================================================================
        /// <summary>
        /// 例外ログ出力処理
        /// </summary>
        /// <param name="ex"></param>
        /// <param name="method">エラーの発生したメソッド</param>
        /// <returns></returns>
        public static void WriteExceptionLog(Exception ex )
        {
            try
            {
                // 初期化されていない場合終了
                if (!_initFlag)
                    return;

                // ファイルサイズチェック
                CheckFileSize();

                // ログファイルオープン
                using (FileStream fs = new FileStream(_logFileName, FileMode.Append, FileAccess.Write))
                {
                    using (StreamWriter sw = new StreamWriter(fs))
                    {
                        // 日時
                        sw.WriteLine(string.Format("[{0:yyyy/MM/dd HH:mm:ss.fff}]", DateTime.Now));

                        // エラー内容
                        sw.WriteLine(ex.ToString());

                        // StreamWriter クローズ
                        sw.Close();
                        sw.Dispose();
                    }

                    // FileStream クローズ
                    fs.Close();
                    fs.Dispose();
                }
            }
            catch
            {
                // 例外は無視
            }        
        }

        #endregion

        #region イベント処理
        #endregion

        #region プライベートプロパティ
        #endregion

        #region プライベートメソッド
        //=====================================================================
        /// <summary>
        /// ファイルサイズチェック処理
        /// </summary>
        /// <remarks>ファイルサイズがオーバーした場合は old に退避します。</remarks>
        private static void CheckFileSize()
        {
            try
            {
                //ファイルが存在する場合
                if (File.Exists(_logFileName))
                {
                    // ファイル情報取得
                    FileInfo fInfo = new FileInfo(_logFileName);

                    // ファイルサイズがオーバーしている場合
                    if (fInfo.Length > MAX_LOG_FILE_SIZE)
                    {
                        // 退避ファイル名
                        string oldFileName = _logFileName + ".old";

                        // すでにファイルが存在する場合削除
                        if (File.Exists(oldFileName))
                            File.Delete(oldFileName);

                        // ファイル移動
                        File.Move(_logFileName, oldFileName);
                    }
                }
            }
            catch
            {
                // 例外は無視
            }
        }
        #endregion
    }
}
